@@ -1,23 +1,27 @@
-/* Clearlooks Gummy style
- * Copyright (C) 2007 Andrea Cimitan
+/* Clearlooks - a cairo based GTK+ engine
+ * Copyright (C) 2007-2008 Andrea Cimitan <andrea.cimitan@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
+ * version 2.1 of the License, or (at your option) any later version.
+ * 
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
+ * 
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Project contact: <gnome-themes-list@gnome.org>
+ *
+ *
+ * This file defines the Clearlooks Gummy style
  *
- * Written by Andrea Cimitan <andrea.cimitan@gmail.com>
  */
+ 
 
 #include "clearlooks_draw.h"
 #include "clearlooks_style.h"
@@ -257,6 +261,7 @@ static void
 clearlooks_gummy_draw_entry (cairo_t                *cr,
                              const ClearlooksColors *colors,
                              const WidgetParameters *params,
+                             const FocusParameters  *focus,
                              int x, int y, int width, int height)
 {
 	const CairoColor *base = &colors->base[params->state_type];
@@ -264,7 +269,7 @@ clearlooks_gummy_draw_entry (cairo_t                *cr,
 	double radius = MIN (params->radius, MIN ((width - 4.0) / 2.0, (height - 4.0) / 2.0));
 
 	if (params->focus)
-		border = colors->spot[2];
+		border = focus->color;
 
 	cairo_save (cr);
 
@@ -281,6 +286,7 @@ clearlooks_gummy_draw_entry (cairo_t                *cr,
 	/* Draw the inner shadow */
 	if (params->focus)
 	{
+		/* XXX Fix this with focus->has_color */
 		clearlooks_set_mixed_color (cr, base, &colors->spot[1], 0.5);
 		ge_cairo_inner_rounded_rectangle (cr, 2, 2, width-4, height-4, MAX(0, radius-1), params->corners);
 		cairo_stroke (cr);
