@@ -1,24 +1,32 @@
-/* Redmond theme engine
+/* Redmond95 - a cairo based GTK+ engine
  * Copyright (C) 2001 Red Hat, Inc.
+ * Copyright (C) 2006 Andrew Johnson <acjgenius@earthlink.net>
+ * Copyright (C) 2006-2007 Benjamin Berg <benjamin@sipsolutions.net>
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
+ * version 2.1 of the License, or (at your option) any later version.
+ * 
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
+ * 
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ *
+ * Project contact: <gnome-themes-list@gnome.org>
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
  *
  * Written by Owen Taylor <otaylor@redhat.com>
  * modified by Andrew Johnson <ajgenius@ajgenius.us>
- */ 
+ * modified by Benjamin Berg <benjamin@sipsolutions.net>
+ *
+ */
+
+
 #include "redmond_gtk2_engine.h"
 #include "redmond_gtk2_drawing.h"
 #include "redmond_gtk2_misc.h"
@@ -38,8 +46,11 @@ do_redmond_draw_masked_fill (cairo_t * cr,
 	cairo_fill(cr);
 
 	ge_cairo_set_color(cr, foreground);
+	cairo_save (cr);
 	cairo_rectangle (cr, x, y, width, height);
+	cairo_clip (cr);
 	cairo_mask (cr, mask->handle);
+	cairo_restore (cr);
 }
   
 /***********************************************
